
__title__ = "syslogcef"

__description__ = "Python library to easily send CEF formatted messages to syslog server"

__url__ = "https://github.com/tristanlatr/syslogcef"

__license__ = "MIT License"

__keywords__ = "syslog cef"

__version__ = "0.1.8"
